# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011-2013 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class ApplicationController < ActionController::Base
  # Acts as Authenticatedプラグインを使用する。
  require "authenticated_system"
  include AuthenticatedSystem

  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  protect_from_forgery # :secret => 'c37e08b75d71b74512e029932a391d84'

  def role_keiri_required
    if !current_user.has_role?(:keiri)
      redirect_to '/account/login'
    end
  end
  private :role_keiri_required
  
  def role_hrd_required
    if !current_user.has_role?(:hrd)
      redirect_to '/account/login'
    end
  end
  private :role_hrd_required
  
  def role_apar_required
    if !current_user.has_role?(:apar)
      redirect_to '/account/login'
    end
  end
  private :role_apar_required

end


#################################################################
# 共通ルーチン

# 10^n倍して四捨五入
def nominal_amount real, exp
  (real.to_f * (10 ** exp) + 0.5).floor
end


def date_from_select(s, name)
  return Date.new(s["#{name}(1i)"].to_i, s["#{name}(2i)"].to_i, s["#{name}(3i)"].to_i)
end


################ 文字コード関係

# UTF-8 to Shift_JIS (cp932)
def utf2sjis s
  if s
    NKF.nkf('-m0 -s -W', s)
  else
    nil
  end
end

def sjis2utf s
  if s
    NKF.nkf('-w -S -m0', s)
  else
    nil
  end
end


# 外貨建の金額を円建てに換算
# @deprecated   Use get_gl_amount().
# @return [Array] 配列[円建ての金額, 為替レート]
def get_yen_amount entry_day, amount, currency
  raise TypeError if !currency.is_a?(Currency)

  # 直近の日付を探して
  cr = CurrencyRate.find :first,
                           :select => 'MAX(date) as date',
                           :conditions => ['currency_id = ? AND date <= ?',
                                           currency.id, entry_day]
  raise "not found currency rate: currency = #{currency.name}, entry_day = #{entry_day}" if !cr || !cr.date

  # 適切なレコードを取得する
  cr = CurrencyRate.find :first,
                           :conditions => ['currency_id = ? AND date = ?',
                                           currency.id, cr.date]
  return [(Float(amount) * cr.rate / 10 ** currency.exponent + 0.5).floor, 
          cr.rate]
end


# 外貨建の金額を機能通貨建てに換算
# @return [Array] 配列[機能通貨建ての金額, 為替レート]
def get_gl_amount company, entry_day, amount, amt_currency
  raise TypeError if !amt_currency.is_a?(Currency)

  # 直近の日付を探して
  cr = CurrencyRate.find :first,
                           :select => 'MAX(date) as date',
                           :conditions => ['currency_id = ? AND date <= ?',
                                           amt_currency.id, entry_day]
  raise "not found currency rate: currency = #{amt_currency.code}, entry_day = #{entry_day}" if !cr || !cr.date

  # 適切なレコードを取得する
  cr = CurrencyRate.find :first,
                         :conditions => ['currency_id = ? AND date = ?',
                                           amt_currency.id, cr.date]
  fr = CurrencyRate.find :first,
                         :conditions => ["currency_id = ? AND date = ?",
                                       company.functional_currency.id, cr.date]

  return [ ((BigDecimal.new(amount.to_s) * cr.rate / fr.rate) / 10 ** amt_currency.exponent + BigDecimal.new("0.5")).floor, 
           (cr.rate / fr.rate) ]
end


def currency_rate entry_day, currency1, currency2
  # 直近の日付を探して
  cr = CurrencyRate.find :first,
                           :select => 'MAX(date) as date',
                           :conditions => ['currency_id = ? AND date <= ?',
                                           currency1.id, entry_day]
  raise "not found currency rate: currency = #{currency1.name}, entry_day = #{entry_day}" if !cr || !cr.date

  # 適切なレコードを取得する
  cr1 = CurrencyRate.find :first,
                          :conditions => ['currency_id = ? AND date = ?',
                                           currency1.id, cr.date]
  cr2 = CurrencyRate.find :first,
                          :conditions => ['currency_id = ? AND date = ?',
                                           currency2.id, cr.date]

  return cr1.rate / cr2.rate
end


# 3桁ごとにカンマを挿入
def format_amount(n, exp)
  n = Float(n) / (10.0 ** exp) if exp != 0
  intpart, exppart = n.to_s.split('.', 2)
  # raise "exp is #{exp}, but n = #{n}" if exp != 0 && exppart && exppart.length > exp
  intstr = intpart.reverse.split(/([0-9]{1,3})/).collect {|d| d == "" ? ',' : d}.join('').reverse.chop
  if exp != 0
    if exp > exppart.length
      sprintf("%s.%s%s", intstr, exppart, "0" * (exp - exppart.length) )
    else
      sprintf("%s.%s", intstr, exppart) # , "0" * (exp - exppart.length))
    end
  else
    sprintf("%s", intstr)
  end
end

