# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


require 'partner_search_box'

# 取引先に紐づける（振込先）銀行口座
class BankAccountsController < ApplicationController
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => 'index' }

  # 一覧画面
  def index
    @bank_accounts = BankAccount.paginate :page => params[:page], :per_page => 20
  end

  def show
    @bank_account = BankAccount.find(params[:id])
  end

  def new
    @bank_account = BankAccount.new
  end

  def create
    @bank_account = BankAccount.new(params[:bank_account])
    if @bank_account.save
      flash[:notice] = 'BankAccount was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @bank_account = BankAccount.find(params[:id])
  end

  def update
    @bank_account = BankAccount.find(params[:id])
    if @bank_account.update_attributes(params[:bank_account])
      flash[:notice] = 'BankAccount was successfully updated.'
      redirect_to :action => 'show', :id => @bank_account
    else
      render :action => 'edit'
    end
  end

  def destroy
    ba = BankAccount.find(params[:id])
    partner = ba.partner
    ba.destroy
    redirect_to :controller => 'partners', :action => 'show', :id => partner
  end
  
  # 紐づける取引先を変更する画面
  def change_partner
    @bank_account = BankAccount.find params[:id]
  end
  
  def change_partner2
    @bank_account = BankAccount.find params[:id]

    @search = PartnerSearchBox.new(params[:search])
    @partners = @search.find_by_text
  end
  
  def change_partner_do
    @bank_account = BankAccount.find params[:id]
    @partner = Partner.find params[:partner]

    @bank_account.partner = @partner
    if @bank_account.save
      flash[:notice] = '銀行口座は正常に更新されました。'
      redirect_to :controller => 'partners', :action => 'show', :id => @partner
    else
      raise
    end
  end

end
