# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# カンパニー設定
class CompaniesController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    if !Company.find(:first)
      redirect_to :action => 'new'
    else
      redirect_to :action => 'list'
    end
  end

  def list
    @company_settings = Company.paginate :page => params[:page],
                                                :per_page => 20
  end

  def show
    @company = Company.find params[:id]
  end

  def new
    if Currency.count == 0
      flash[:alert] = "通貨が一つもありません。先に通貨マスタ画面で通貨を登録してください。" 
      redirect_to :controller => "currencies", :action => "index"
    else
      @company = Company.new
    end
  end


  # 作成
  def create
    @company = Company.new params[:company]
    today = Date.today
    @company.attributes = {
      :fixed_day => today,
      :fixed_gl_day => today,
    }
    @company.si_counter = 1 if @company.si_counter.to_i == 0

    begin
      @company.save!
    rescue ActiveRecord::RecordInvalid
      render :action => 'new'
      return
    end

    flash[:notice] = 'カンパニーを作成しました。'
    redirect_to :action => 'show', :id => @company
  end

  
  # マスタ系の編集 (画面)
  def edit
    @company = Company.find params[:id]
    @divisions = Division.find :all, 
           :conditions => ["end_day IS NULL AND company_id = ?", @company.id],
           :order => "division_code"
    if @divisions.count == 0
      flash[:alert] = "このカンパニーの部門が一つもありません。まず会計部門/コストセンタを登録してください。"
      redirect_to :controller => "control_divisions", :action => "index"
    end
  end
  

  def update
    @company = Company.find params[:id]
    image = params[:company][:logo_image]
    params[:company].delete :logo_image
    @company.attributes = params[:company]
    @company.logo_image = image.read if !image.blank?

    begin
      Company.transaction do 
        if !@company.company_setting
          cs = CompanySetting.new :company_id => @company.id,
                       :bank_division_id => params[:bank_division_id],
                       :fallback_division_id => params[:fallback_division_id]
          cs.save!
        else
          @company.company_setting.attributes = {
            :bank_division_id => params[:bank_division_id],
            :fallback_division_id => params[:fallback_division_id]
          }
          @company.company_setting.save!
        end
        @company.save!
      end
    rescue ActiveRecord::RecordInvalid
      @divisions = Division.find :all, 
           :conditions => ["end_day IS NULL AND company_id = ?", @company.id],
           :order => "division_code"
      render :action => 'edit'
      return
    end

    flash[:notice] = 'カンパニーを更新しました。'
    redirect_to :action => 'show', :id => @company
  end
  

  # 伝票表示日, 仕訳生成確定日の編集 (画面)
  def gl_fix
    @company_setting = Company.find params[:id]
  end
  
  def gl_fix2
    @company_setting = Company.find params[:id]
    if @company_setting.update_attributes(params[:company_setting])
      flash[:notice] = 'company_setting was successfully updated.'
      redirect_to :action => 'show', :id => @company_setting
    else
      render :action => 'gl_fix'
    end
  end

  def destroy
    Company.transaction do
      company = Company.find params[:id]
      company.company_setting.destroy
      company.destroy
    end

    flash[:notice] = 'カンパニーを削除しました。'
    redirect_to :action => 'list'
  end
  
  # 画像データ
  def logo_image
    r = Company.find params[:id]
    send_data r.logo_image, :type => "image/png"
  end
end
