# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


require "validating_base"

class DivSearchBox < ValidatingBase
  attr_accessor :name
end


# 会計部門
# 組織コードを含む前提
class DivisionsController < ApplicationController
  before_filter :login_required

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, 
         :only => [ :destroy, :create, :update, :update_addable ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end

  # 一覧表示
  def list
    @divisions = Division.paginate :page => params[:page], :per_page => 25,
                                   :order => 'division_code'
    @page = params[:page]
  end


  def search
    @search = DivSearchBox.new params[:search]
    if @search.name.to_s.strip == ""
      @search.errors.add "name", "検索テキストを指定してください。"
      index
      return
    end

    text = "%" + normalize_kc(@search.name.strip).upcase + "%"
    @divisions = Division.find :all,
                      :conditions => ["upper(name) LIKE ? OR kana_name LIKE ?",
                                      text, text],
                      :order => "division_code"
  end


  def show
    @division = Division.find(params[:id])
  end


  def new
    err = ControlDivision.roots.empty?
    ControlDivision.roots.each do |root|
      err = true if root.direct_children.empty?
    end

    if err
      flash[:alert] = "コストセンタがありません。先にコストセンタを設定してください。"
      redirect_to :controller => "control_divisions", :action => "index"
      return
    end

    @division = Division.new
  end
  

  # 作成
  def create
    @division = Division.new params[:division]
    @division.end_day = nil if !params[:disable]
    Division.transaction {
      if @division.save
        params[:ctr].each {|cid|
          cdiv = ControlDivision.find cid
          ControllersDivision.create :control_division => cdiv, :division => @division
        }
        flash[:notice] = 'Division was successfully created.'
        redirect_to :action => 'list'
      else
        render :action => 'new'
      end
    }
  end
  
  # 編集画面
  def edit
    @division = Division.find(params[:id])
    @page = params[:page]
  end

  def update
    @division = Division.find params[:id]
    @division.attributes = params[:division]
    @division.end_day = nil if !params[:disable]
    Division.transaction {
      if @division.save
        ControllersDivision.delete_all ['division_id = ?', @division.id]
        params[:ctr].each {|cid|
          cdiv = ControlDivision.find cid
          ControllersDivision.create :control_division => cdiv, :division => @division
        }
        flash[:notice] = 'Division was successfully updated.'
        redirect_to :action => 'show', :id => @division
      else
        render :action => 'edit'
      end
    }
  end
  
  # 削除する
  def destroy
    div = Division.find params[:id]
    raise if !div
    # コストセンタから切り離す
    ControllersDivision.delete_all ['division_id = ?', div.id]
    div.destroy
    redirect_to :action => 'list'
  end
  
  # 購買伝票で使用可にするかの一覧
  def select_addable
    @divisions = Division.find(:all,
                       :conditions => 'end_day is NULL',
                       :order => 'division_code')
  end
  
  # 更新する
  def update_addable
    list = params[:ss]
    Division.transaction {
      Division.find(:all).each {|r|
        orig_active = r.purchase_active
        r.purchase_active = list.index(r.id.to_s) ? 1 : 0   # activeでチェックを入れる
        r.save! if orig_active != r.purchase_active
      }
    }
    redirect_to :action => 'select_addable'
  end
end
