# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 出金伝票
class PaymentSlipsController < ApplicationController
  before_filter :login_required
  before_filter :role_keiri_required

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post,
         :only => [ :destroy, :create, :sign_off, :add_lines, 
                    :destroy_line, :update_cash, :cancel_slip ],
         :redirect_to => { :controller => "receivings", :action => :index }

  verify :method => :put, :only => [ :update ],
         :redirect_to => { :controller => "receivings", :action => :index }

=begin
  # GET /payment_slips
  # GET /payment_slips.xml
  def index
    @payment_slips = PaymentSlip.all

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @payment_slips }
    end
  end
=end


  # 詳細表示
  def show
    @payment_slip = PaymentSlip.find params[:id]
  end


  # 伝票印刷
  def print
    @payment_slip = PaymentSlip.find params[:id]
  end


  def new
    @company = Company.find params[:company]
    @payment_slip = PaymentSlip.new
  end


  # GET /payment_slips/1/edit
  def edit
    @payment_slip = PaymentSlip.find params[:id]
  end


  def edit_cash
    @payment_slip = PaymentSlip.find params[:id]
  end


  # 新規作成
  def create
    @payment_slip = PaymentSlip.new params[:payment_slip]
    @payment_slip.attributes = {
      :state => 0,
      :create_user_id => current_user.id,
      :company_id => params[:company]
    }

    if @payment_slip.save
      redirect_to :action => "show", :id => @payment_slip, 
                  :notice => 'PaymentSlip was successfully created.'
    else
      render :action => "new"
    end
  end


  # PUT /payment_slips/1
  # PUT /payment_slips/1.xml
  def update
    @payment_slip = PaymentSlip.find params[:id]

    if @payment_slip.update_attributes(params[:payment_slip])
      redirect_to( {:action => "show", :id => @payment_slip}, 
                  :notice => 'PaymentSlip was successfully updated.' )
    else
      render :action => "edit"
    end
  end


  # 伝票の削除 (実行)
  def destroy
    @payment_slip = PaymentSlip.find params[:id]
    PaymentSlip.transaction do
      cond = ["payment_slip_id = ?", @payment_slip.id]
      Payment.delete_all cond
      PaymentBank.delete_all cond
      RsLog.delete_all cond
      @payment_slip.destroy
    end

    flash[:notice] = "出金伝票を削除しました。"
    redirect_to :controller => "receiving_slips", :action => "list", 
                                        :company => @payment_slip.company
  end


  # 支払う債務を選ぶ
  def remain
    @payment_slip = PaymentSlip.find params[:id]
    @currency = Currency.find params[:currency]
    @company = @payment_slip.company

    @due_day = params[:due] ? date_from_select(params[:due], 'day') : Date.today + 5

    # 輸入消費税の考慮を忘れずに
    @invoices = Invoice.find :all,
            :conditions => [<<EOF, @due_day, @company.id, @currency.id],
                (invoices.bank_account_id IS NULL) AND 
                (state <= 20) AND payment_state <> 20 AND 
                (due_day <= ?) AND company_setting_id = ? AND
                currency_id = ? AND
amount_total <> (SELECT coalesce(sum(payments.amount), 0) FROM payments 
                     LEFT JOIN payment_slips ON payments.payment_slip_id = payment_slips.id
                     WHERE payment_slips.state <> 30 AND invoices.id = payments.invoice_id)
EOF
            :order => "partner_id, buy_day"
  end


  # 債務の消し込み行を追加(実行)
  def add_lines
    @payment_slip = PaymentSlip.find params[:id]

    payments = []
    currency = nil
    params[:inv].each do |key, r|
      next if r[:amount].blank?
      currency = Invoice.find(key).currency

      # 請求書単位での金額
      amount = nominal_amount r[:amount], currency.exponent

      # 科目単位に分解する
      remain_amounts = Invoice.find(key).remain_ac_amounts
      remain_amounts.each do |crac_id, remain|
        # クレジットの場合はremainがマイナスになっている
        if remain != 0
          pay_amount = remain >= 0 ? [remain, amount].min : 
                                       [remain, amount].max
          payments << (Payment.new :payment_slip_id => @payment_slip.id,
                                 :invoice_id => key,
                                 :account_title_id => crac_id,
                                 :amount => pay_amount,
                                 :currency_id => currency.id )
          amount -= pay_amount
          break if amount == 0
        end
      end
    end

    Payment.transaction do 
      amount_total = 0
      payments.each do |r|
        r.save!
        amount_total += r.amount
      end
=begin
債務の通貨の口座がないことがある。無理筋
      if @payment_slip.payment_banks.empty?
        bank = nil
        OurBank.find(:all).each do |b|
          if b.currency.code == currency.code
            bank = b
            break
          end
        end
        r = PaymentBank.new :payment_slip => @payment_slip,
                        :our_bank => bank,
                        :cash_amount => amount_total,
                        :inv_amount => amount_total
        r.save!
      end
=end
    end # transaction

    flash[:notice] = "債務の消し込み行を追加しました。"
    redirect_to :action => "show", :id => @payment_slip
  end # add_lines()


  # 債務の消し込み行を1行削除
  def destroy_line
    @payment_slip = PaymentSlip.find params[:id]
    count = @payment_slip.payments.size
    line = Payment.find params[:line]

    Payment.transaction do 
      line.destroy
      if count == 1
        PaymentBank.delete_all ["payment_slip_id = ?", @payment_slip.id]
      end
    end # transaction

    flash[:notice] = "債務の消し込み行を削除しました。"
    redirect_to :action => "show", :id => @payment_slip
  end


  # 出金を更新
  def update_cash
    @payment_slip = PaymentSlip.find params[:id]
    inv_currency = @payment_slip.payments.first.invoice.currency

    lines = 5

    PaymentSlip.transaction do
      PaymentBank.delete_all ["payment_slip_id = ?", @payment_slip.id]

      (0..(lines - 1)).each do |lineno|
        pa = params["payment_bank#{lineno + 1}"]
        if pa[:cash_amount].to_f != 0.0
          bank = OurBank.find pa[:our_bank]
          pb = PaymentBank.new :payment_slip => @payment_slip,
                   :our_bank => bank,
                   :cash_amount => nominal_amount(pa[:cash_amount], bank.currency.exponent)
          if bank.currency.id == inv_currency.id
            pb[:inv_amount] = nominal_amount(pa[:cash_amount], inv_currency.exponent)
          else
            pb[:inv_amount] = nominal_amount(pa[:inv_amount], inv_currency.exponent)
          end
          pb.save!
        end
      end
    end # transaction

    flash[:notice] = "出金を更新しました。"
    redirect_to :action => "show", :id => @payment_slip
  end


  # 経理承認 (実行)
  def sign_off
    @payment_slip = PaymentSlip.find params[:id]
    PaymentSlip.transaction do 
      if @payment_slip.state < 10
        log = RsLog.new :payment_slip => @payment_slip,
                        :kind => RsLog::SIGN_OFF,
                        :remarks => "",
                        :done_by => current_user,
                        :done_at => Time.now
        log.save!
        @payment_slip.state = 10
        @payment_slip.save!
      end
    end # transaction
    flash[:notice] = "出金伝票を承認しました。"
    redirect_to :action => "show", :id => @payment_slip
  end


  def cancel_slip
    payment_slip = PaymentSlip.find params[:id]
    company = payment_slip.company
    PaymentSlip.transaction do
      payment_slip.state = 30
      payment_slip.save!

      log = RsLog.new :payment_slip => payment_slip,
                      :kind => RsLog::CANCEL,
                      :remarks => "",
                      :done_by => current_user,
                      :done_at => Time.now
      log.save!
    end # of transaction
    flash[:notice] = "出金伝票をvoidしました。"
    redirect_to :controller => "receiving_slips",
                :action => "list", :company => company
  end
end
