# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 取引先（仕入先）
class Partner < ActiveRecord::Base
  has_many :bank_accounts
  has_many :invoices
  has_many :sales_invoices
  has_many :attach_file_refs
  belongs_to :created_by, :foreign_key => 'created_by', :class_name => 'User'
  belongs_to :updated_by, :foreign_key => 'updated_by', :class_name => 'User'
  # has_many :attach_files, :through => :attach_file_refs
  
  validates_presence_of :name
  validates_presence_of :kana_name
  validates_uniqueness_of :zaimu_id
  validates_format_of :hq_zip, :with => /^[0-9-]*$/

  validates_uniqueness_of :tdb_code, :allow_nil => true

  before_save :normalize_string
  
  def normalize_string
    self.name = normalize_kc(name)
    self.kana_name = normalize_kc(kana_name).gsub(/[ 　\t\r\n・]/, '')
    self.remarks = normalize_kc remarks
    self.hq_zip = normalize_kc hq_zip
    self.hq_addr = normalize_kc hq_addr
  end

  # ファイルの中身を取得しない
  def attach_files
    AttachFile.find :all,
           :select => 'a.id, a.original_filename, a.content_type, a.attach_kind_id, a.remarks, a.division_id, a.create_user_id, a.created_at',
           :from => 'attach_files AS a',
           :joins => 'JOIN attach_file_refs ON a.id = attach_file_refs.attach_file_id',
           :conditions => ['attach_file_refs.partner_id = ?', self['id']]
  end

  def all_bank_accounts activeonly = true
    ba = BankAccount.find :all,
          :conditions => ["partner_id = ?" + (activeonly ? " AND active" : ""),
                          self.id]
    ba += EnBankAccount.find( :all,
                       :conditions => ["partner_id = ?", self.id] )
    return ba
  end
end
