# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 入金の1行
class Receiving < ActiveRecord::Base
  # 親
  belongs_to :receiving_slip

  # いずれか一方
  belongs_to :sales_invoice
  belongs_to :petty_expense

  #belongs_to :our_bank
  belongs_to :currency

  belongs_to :created_by, :class_name => 'User', :foreign_key => 'created_by'

  # credit noteがある。 => 正値とは限らない
  # validates_numericality_of :amount, :greater_than_or_equal_to => 0

  validates_numericality_of :charge, :greater_than_or_equal_to => 0


  # 貸方だけ埋める
  def make_partial_entry company, entry_day
    raise TypeError if !company.is_a?(Company)
    raise TypeError if !entry_day.is_a?(Date)

    gl_amt, gl_rate = get_gl_amount company, entry_day, amount, currency

    if sales_invoice
      # 取引先
      e = {
        :cr_account_title => sales_invoice.receivable_account,
        :cr_division => nil,
        :cr_partner_code => sales_invoice.partner.zaimu_id,
        :amount => gl_amt,
        :vat => nil,
        :remarks => [sales_invoice.partner.name, 
                     sales_invoice.si_journals.first.remarks,
                     "入金"].join("/")
      }
    else
      # 社員
      raise "internal error" if !petty_expense

      e = {
        :cr_account_title => AccountTitle.find_by_account_code(170), # TODO:
        :cr_division => nil,
        :cr_partner_code => nil,  # TODO: 社員コード
        :amount => gl_amt,
        :vat => nil,
        :remarks => ["#{petty_expense.user.name} (#{petty_expense.user.worker_code})",
                     "社員未収入金", "入金"].join("/")
      }
    end

    [e, gl_rate]
  end
end
